#!/bin/python
##############################################################
#    Yasuko Matsubara 
#    Date: 11-10-2014
##############################################################
import sys
import pylab
import numpy as np
import math as M
import tool
class EXO:
    def __init__(self, d, n, Pp):
        self.d = d; self.n = n; self.Pp=Pp;
        self.resetEXO()
    def resetEXO(self):
        _allocEXO(self, self.d, self.n, self.Pp)
    def getE(self, n):
        return _exo(self, n)
    def readParams(self, fn):
        _readParams(self, fn)
    def printParams(self, fn):
        _printParams(self, fn)

#--------------------------------------------#
#
#    PRIVATE FUNCS
#
#--------------------------------------------#
def _allocEXO(exo, d, n, Pp):
    exo.maxk=d; exo.optk=0;
    exo.B=np.zeros((exo.maxk, Pp))
    exo.W=np.zeros((d, exo.maxk))

def _exo(exo, n):
    d = exo.d; Pp = exo.Pp;
    B = exo.B; W = exo.W;
    optk=exo.optk;
    E = np.zeros((d,n))
    for i in range(0,d):
        for t in range(0,n):
            E[i][t] = 0.0;
            # for each external acivity
            for k in range(0,optk):
                E[i][t] += W[i][k]*B[k][np.mod(t,Pp)]
    return E


def _printEXO(exo):
    print "----------------------"
    print "# of exos: ", exo.optk
    print "Pp: ", exo.Pp
    print exo.W
    print "----------------------"

def _readParams(exo, fn):
    fn=fn+'_exo'
    print "read exo model parameters (W, B) ..."
    try:
        f = open(fn, 'r')
    except:
        tool.warning('readParams: cannot open file'+fn)
        exo.optk=0; return;
    d    = int(f.readline().rstrip().split(' ')[1])
    Pp   = int(f.readline().rstrip().split(' ')[1])
    optk = int(f.readline().rstrip().split(' ')[1])
    f.close()
    # if paramset is not appropriate
    if(exo.d!=d or exo.Pp!=Pp):
        tool.warning('exo params error'); return;
    exo.d=d; exo.Pp=Pp; exo.optk=optk
    exo.B=pylab.loadtxt(fn+'_B', ndmin=2)
    exo.W=pylab.loadtxt(fn+'_W', ndmin=2)
    (kB, Pp) = np.shape(exo.B)
    (d, kW)  = np.shape(exo.W)
    if(kB!=kW):
        tool.error("B[%s][%s], W[%s][%s]"%(kB, Pp, d, kW))
    #exo.optk = kB

def _printParams(exo, fn):
    if(fn==''):
        _printEXO(exo); return;
    fn = fn + 'model_exo'
    f = open(fn, 'w')
    sys.stdout = f
    print "d= %d"  %exo.d
    print "Pp= %d" %exo.Pp
    print "k= %d"  %exo.optk
    sys.stdout = sys.__stdout__
    f.close()
    optk=exo.optk
    np.savetxt(fn+'_B', exo.B[0:optk,:], fmt='%.3f', delimiter="\t")
    np.savetxt(fn+'_W', exo.W[:,0:optk], fmt='%.3f', delimiter="\t")
    np.savetxt(fn+'_E', exo.getE(exo.Pp), fmt='%.3f', delimiter="\t")


#---------------#
#     main      #
#---------------#
if __name__ == "__main__":
 
    d=2; n=200; Pp=52
    exo=EXO(d,n, Pp)
    exo.B[0]=np.random.rand(Pp)
    exo.B[0]=pylab.loadtxt("UNUSED/tmp_dat/E")
    exo.W[0][0]=1.0
    exo.W[1][0]=0.9
    exo.optk=1
    #exo.readParams("../output/google/s_1112/u_model")

    exo.printParams('')
    E = exo.getE(n)
    print 'save: ./UNUSED/tmp_dat/E.txt'
    np.savetxt('UNUSED/tmp_dat/E.txt', E, fmt='%.4f', delimiter="\t")


